# File: license_config.awk
#       Generate site specific license.lic file
#
# Update log
#
# Oct  4 2023 JAB: Replace an occurrence of license.demo with license_demo.lic
#                  and an occurrence of directory name rlm-*.* with
#                  "TotalView's license directory"
# Oct  3 2023 JAB: Replace an occurrence of license.dat with license.lic
# Sep  6 2023 JAB: Created from same-named FNP script 
# Dec 31 2020 JAB: [TVT-32152] Configure_License FNP should support
#                  node-locked licenses
# Dec 29 2020 JAB: Changed email to perforce.com
# Jan 12 2016 JAB: [TVT-20414] Start using VENDOR instead of DAEMON in
#                  license.dat
# Jan 12 2016 JAB: [TVT-20413] Add USE_SERVER to license.dat
# Jan 10 2014 JAB: [TVT-15858] Support for FlexNet lmhostid 11.11 which
#                  might return multiple host IDs on some systems
# Mar 31 2011 NYP: Changed email to roguewave.com
# Dec  7 2007 NYP: Updated experation dates to 2008 (From AVS)
# Jul 14 2005 JGC: Fixed Bug 8315. Fixed by making the script match
#                  DEMO before HOSTID.
# Jun 17 2005 JGC: Fixed license munging problem on HOSTID
#                  Fixed some issues dealing with path names with spaces
# Jul 30 2003 BLC: Updated to be compatible with new license delivery
#                  format.  Error output improved for support.
#
#------------------------------------------------------------------------
#

BEGIN {
    server_hostname=""
    server_hostid=""
    daemon_port=""
    daemon_path=""
    optfile_path=""
    endhost=0
    header=0
    beginlic=0
    foundlic=0
    founddemo=0
    beginline=""
    LC="abcdefghijklmnopqrstuvwxyz"
    UC="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    print_header=1
}

(endhost == 0) && ($1 == "<hostname>") {server_hostname=$2; next}
(endhost == 0) && ($1 == "<portno>")   {daemon_port=$2; next}

(endhost == 0) && ($1 == "<daemon>")    { 

  # This is to account for spaces in path names
  split($0, a )
  p=index( $0, a[2])
  daemon_path=substr($0, p, length($0))
  next  
   
}

(endhost == 0) && ($1 == "<optfile>")  { 

  # This is to account for spaces in path names
  split($0, a )
  p=index( $0, a[2])
  optfile_path=substr($0, p, length($0))
  next

}
(endhost == 0) && ($1 == "<hostid>")   {

#  print $2 | "cat 1>&2"
  # Store the host IDs in an array
  split($2, hid_arr, ":")

  for (key in hid_arr) {

    ###################################
    # BEGIN inline tolower() function #
    ###################################
    out=""
    for(i=1; i<=length(hid_arr[key]); i++) {
      char=substr(hid_arr[key],i,1)
      j=index(UC,char)
      if (j > 0) {
        out = out substr(LC,j,1)
      } else {
        out = out char
      }
    }
    hid_arr[key] = out
    ###################################
    # END   inline tolower() function #
    ###################################

    ++hid_arr_sz
  }
  next
}

/## END HOST/ {endhost=1; next}

(header == 0 && endhost == 1) {

  # Check the host information input
  if (server_hostname == "") {
    print "Error: Undefined server hostname." | "cat 1>&2"
    print "" | "cat 1>&2"
    exit 1
  }

  if (hid_arr_sz == 0) {
    print "Error: Undefined server hostid." | "cat 1>&2"
    exit 1
  }

  if (daemon_port == "") {
    print "Error: Undefined server daemon port." | "cat 1>&2"
    exit 1
  }

  if (daemon_path == "") {
    print "Error: Undefined server daemon path." | "cat 1>&2"
    exit 1
  }

#  lmgrd doesn't like spaces in the opt file path in the license file
#  So we don't use the options :(

#  if (optfile_path == "") {
#    print "Error: Undefined server option file path." | "cat 1>&2"
#    exit 1
#  }

  # Do not execute this block any more
  header=1
}

/## BEGIN.*KEY/ {

  # Store the BEGIN key header string.  Do not print the line 
  # until we've verified a hostid match for the license key.
  beginline=$0 
  next
}

/DEMO/ {

  # Record that a DEMO license key was found.  It will not appear in the 
  # resulting license file.  Use this information to return an appropriate 
  # error message if no license keys were found but a DEMO key was found.
  founddemo=1
  next
}

/#*[ \t\v]*HOSTID/ && (beginlic == 0) {

  # Write a comment out so lmgrd doesn't break
  if ( $0 !~ /^#/ ) {
    $0 = "#" $0
  }

  string=$0
  ###################################
  # BEGIN inline tolower() function #
  ###################################
  out=""
  for(i=1; i<=length(string); i++) {
    char=substr(string,i,1)
    j=index(UC,char)
    if (j > 0) {
      out = out substr(LC,j,1)
    } else {
      out = out char
    }
  }
  string = out
  ###################################
  # END   inline tolower() function #
  ###################################
  lowercase_key_hostid=string

  # Try to find a matching host ID in the host ID array
  if (hid_match == 0) {
    for (key in hid_arr) {
      if (index(lowercase_key_hostid, hid_arr[key])) {
        server_hostid = hid_arr[key]
	hid_match = 1
        break;
      }
    }
  }

  if (hid_match == 0 || !index(lowercase_key_hostid, server_hostid)) {
    # This license key block does not have a HOSTID that corresponds 
    # with this license server.  This error should be fatal.
    print "Error: Key rejected due to license server host ID mismatch." | "cat 1>&2"
    exit 1
  }

  if (print_header == 1) {
    # Begin by printing the server and daemon information 
    print "HOST", server_hostname, server_hostid, daemon_port
    print "ISV prqa", daemon_path, optfile_path
    print_header = 0
  }

  if (beginline != "") {
    # This license key block has a BEGIN header and a HOSTID that 
    # corresponds with this license server.
    beginlic=1

    # Print the stored license key header comment and reset.
    print beginline
    beginline=""

    # Print the HOSTID=xxxxxxxx comment line.
    print $0
    next
  }
  else {
    # The license key block is missing the BEGIN header.  Reject the key 
    # and provide a useful error message.  
    print "Error: A key was rejected due to missing \"## BEGIN TOTALVIEW ....\" license key header string." | "cat 1>&2"
    beginlic=0
    next
  }
}

/LICENSE/ {
  if (beginlic) {
    print $0
    foundlic=1
    next
    }
  }

/## END.*KEY/ {
  if (beginlic) {
    print $0

    # Reset beginlic and scan for the next key header
    beginlic=0
    next  
  }
}

## every line
{ if (beginlic) print $0 }

END {

  if(founddemo && !foundlic) {
  
    # No permanent license keys were found and DEMO license
    # key lines were found.
    print "Error: No permanent license keys were found." | "cat 1>&2"
    print "Error: " | "cat 1>&2"
    print "Error: DEMO license strings were found but cannot be served by the" | "cat 1>&2"
    print "Error: license manager.  To use a DEMO license, save the text in a" | "cat 1>&2"
    print "Error: file named \"license_demo.lic\" within TotalView's license directory" | "cat 1>&2"
    print "Error: and run TotalView." | "cat 1>&2"
    exit 1
  }
  else if(!founddemo && !foundlic) {

    # No permanent license keys were found and no DEMO strings were found.
    print "Error: No permanent license keys were found." | "cat 1>&2"
    print "Error: " | "cat 1>&2"
    print "Error: Please review the contents of your license.src file.  The format of"      | "cat 1>&2"
    print "Error: each license key should be similar to the following example."             | "cat 1>&2"
    print "Error: "                                                                         | "cat 1>&2"

    print "Error: ## BEGIN PERFORCE TOTALVIEW KEY -- XXXXXXXXXXXXXXXXXXXXXXXXXXXXX"          | "cat 1>&2"
    print "Error: ## HOSTID=<HOSTID>"                                                        | "cat 1>&2"
    print "Error: LICENSE prqa totalview_teams YYYY.MMDD permanent X start=D-mmm-YYYY"       | "cat 1>&2"
    print "Error:   issuer=XXX...XX platforms=XXX...XX customer=XXX...XX"                    | "cat 1>&2"
    print "Error:   contract=XXX...XX issued=D-mmm-YYYY akey=XXX...XX _ck=XXX...XX"          | "cat 1>&2"
    print "Error:   sig=XXX...XX"                                                            | "cat 1>&2"
    print "Error: ..."                                                                       | "cat 1>&2"
    print "Error: ## END PERFORCE TOTALVIEW KEY -- XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"         | "cat 1>&2"
    print "Error: "                                                                          | "cat 1>&2"

    print "Error: NOTE: The number of LICENSE lines and feature names will depend"          | "cat 1>&2"
    print "Error:       on the actual license key configuration."                           | "cat 1>&2"
    print "Error: "                                                                         | "cat 1>&2"
    print "Error: If you have additional questions or require assistance installing"        | "cat 1>&2"
    print "Error: your license key(s), send email to license@perforce.com  Please include"    | "cat 1>&2"
    print "Error: copies of your license.src and/or license.lic files in your message."     | "cat 1>&2"
    exit 1
  }
  else {
    exit 0
  }
}
